
####ESTIMATION OF CONCENTRATION USING THE INVERSE-VARIANCE WEIGHTED METHOD
####Cheung YB et al. Statistical estimation of antibody concentration using multiple dilutions
####Journal of Immunological Methods

##install two R packages 
library(MASS)
library(stats)


###set the working directory 
setwd("F:dilution\\data")

##including user-written sub-routines from file subroutine_MulDiL.r
source("subroutine_MulDiL.r")

####read data set into R

##Data set in .csv format, containing data from both standard samples and test samples. First row of the data set are the variable names
##Below list the required variables and the corresponding descriptions
##Variable name    Format            Variable description
##TestSample       Binary (0/1)      1 if for test samples and 0 if for standard samples
##ID               Numerical         Sample identifier
##plate            Numerical         plate identifer
##dilution         Numerical         reciprocal of the dilution factor
##standard         Numerical         concentration in the standard sample
##od               Numerical         observed optical density based on the standard samples, or the average of the observed optical density over the replications of the test sample

data_samples=read.csv("example_Data.csv", header=TRUE) 


####sd_rep: number of replications for each standard sample [user-specified value, must be greater than 1, same across all standard samples]
####K: number of dilutions for each test sample [user-specified value, must be greater than 1, same across all test samples]
####r0:  number of replications for each test sample [user-specified value, not less than 1, same across all test samples]
####alpha: 100(1-alpha)% confidence interval for the estimated concentration
sd_rep=2
K=4
r0=2 
alpha=0.05

####
##############################ESTIMATION OF RESPONSE MODEL PARAMETERS, USING DATA FROM STANDARD SAMPLES WITH KNOWN CONCENTRATION 
##extracting data information related to the standard samples; to be used for estimation of the response model 
SD_data=data_samples[which(data_samples$TestSample==0),]

##plate_ID:  list of plate identifiers for the standard samples  
##plate_N:   total of number of plates for the standard samples 
plate_ID=as.numeric(names(table(SD_data$plate)))
plate_N=length(plate_ID)

###estimation of the six parameters (b1,b2,b3,b4,theta, sigma) involved in the response model
##(b1,b2,b3,b4): the four-parameter logistic model for the mean response
##(theta, sigma): the two parameters related to the response variance, whereas the response variance is a power of mean (POM) response 

#create a plate_N by 7 matrix for the plate-specific parameter estimates, each row =(plate_ID, b1,b2,b3,b4,theta, sigma)
est_POM=matrix(0,plate_N,7)

####plate-specific parameter estimation starts....
for(i in 1:plate_N)
{
platei=plate_ID[i]
conc_cal_rep=SD_data$standard[which(SD_data$plate==platei)]
od_cal=SD_data$od[which(SD_data$plate==platei)]
stdcrvdata=data.frame(conc_cal_rep, od_cal)

#initialize parameter estimates
sort_idx=sort(conc_cal_rep,index.return=TRUE)$ix
od_dil_cal=matrix(od_cal[sort_idx],ncol=sd_rep,byrow=TRUE)
b1_0=mean(od_dil_cal[1,])
b2_0=mean(od_dil_cal[nrow(od_dil_cal),])
b3_0=mean(conc_cal_rep)

log_s=log(apply(od_dil_cal,1,sd)+0.001)
log_yavg=log(apply(od_dil_cal,1,mean))
lm_theta=lm(log_s~log_yavg)
theta_0=coef(lm_theta)["log_yavg"]

#parameter estimation of (b1,b2,b3,b4,theta) via generalized least squares method, implemented by the user-written subroutine GLS_POM
result_POM=GLS_POM(stdcrvdata,b1_0,b2_0,b3_0,theta_0)
est_POM[i,-7]=c(platei, result_POM)

###parameter estimation of sigma: the standard deviation of the normally-distributed residuals  
b1_est=result_POM[1]
b2_est=result_POM[2]
b3_est=result_POM[3]
b4_est=result_POM[4]
theta_est=result_POM[5]
pred_fit=b2_est+(b1_est-b2_est)/(1+exp(b4_est*(log(conc_cal_rep)-log(b3_est))))
tmp1=(od_cal-pred_fit)^2/(pred_fit^(2*theta_est))
est_POM[i,7]=sqrt(sum(tmp1)/(length(conc_cal_rep)-4))
}

####plate-specific parameter estimation ends....

#print the plate-specific parameter estimates 
print(est_POM)


##############################ESTIMATION OF UNKNOWN CONCENTRATIONS IN THE TEST SAMPLES 
##extracting data information related to the test samples, for which the concentrations are to be estimated 
TEST_data_unsorted=data_samples[which(data_samples$TestSample==1),]

##sort the data according to the variable id, so that data with same id appear in adjcent rows
sort_idx=sort(TEST_data_unsorted$id, index.return = TRUE)$ix
TEST_data=TEST_data_unsorted[sort_idx,]

##conc_est: a column vector containing the concentration estimate for the original test samples based on each diluted test samples
conc_est=rep(0,nrow(TEST_data))
for(i in 1:nrow(TEST_data))
{
#identify the row in the parameter estimate matrix est_POM that corresponds to the same plate containing the test sample
idx_mat=which(est_POM[,1]==TEST_data$plate[i])

#extract the plate-specific parameter estimates for b1,b2,b3,b4 
b1=est_POM[idx_mat,2]
b2=est_POM[idx_mat,3]
b3=est_POM[idx_mat,4]
b4=est_POM[idx_mat,5]

#obtain the dilution-specific estimate for the concentration in the original test sample by inversion of the standard curve
conc_est[i]=TEST_data$dilution[i]*INV_SDcurve(TEST_data$od[i],b1,b2,b3,b4)
}

##adding the conc_est to the TEST_data data set as the last column
TEST_data=cbind(TEST_data,conc_est)


########################################################################
##############inverse-variance weighted estimate for concentration
########################################################################
#######store data information into matrix format, one row corresponding to one original test sample
od_mat=matrix(TEST_data$od, ncol=K, byrow=TRUE)
conc_mat=matrix(conc_est,ncol=K,byrow=TRUE)
dilution_mat=matrix(TEST_data$dilution, ncol=K, byrow=TRUE)
subject_mat=matrix(TEST_data$id, ncol=K, byrow=TRUE)[,1]
plate_mat=matrix(TEST_data$plate, ncol=K, byrow=TRUE)[,1]

n=length(subject_mat)

##estWtAvg_IV: inverse-variance weighted estimate for each original test sample
##var_lnWtAvg_IV: variance estimate for the inverse-variance weighted estimate [on the log-scale]
##Var_logx: variance estimate for dilution-specific concentration estimate
##[Lower_CI_WtAvg_IV, Upper_CI_WtAvg_IV]: 100(1-alpha)% confidence interval for the estimated concentration for each original test sample

estWtAvg_IV=var_lnWtAvg_IV=Lower_CI_WtAvg_IV=Upper_CI_WtAvg_IV=rep(0,n)
Var_logx=matrix(0,n,K)

for(i in 1:n)
{
#extracting parameter estimates for the response model correponding to the plate that this test sample was tested on
platei=plate_mat[i]
idx_mat=which(est_POM[,1]==platei)
b1_est=est_POM[idx_mat,2]
b2_est=est_POM[idx_mat,3]
b3_est=est_POM[idx_mat,4]
b4_est=est_POM[idx_mat,5]
theta_est=est_POM[idx_mat,6]
sigma_est=est_POM[idx_mat,7]

#extracting data of the standard samples on the same plate as the test sample
conc_cal_rep=SD_data$standard[which(SD_data$plate==platei)]
od_cal=SD_data$od[which(SD_data$plate==platei)]

#removing any standard samples with true concentration=0
idx_rm=which(conc_cal_rep==0)
conc_cal_rep=conc_cal_rep[-idx_rm]
od_cal=od_cal[-idx_rm]

####estimation of conconcentration
#variance matrix for the parameter estimates in the four-parameter logistic model for the mean response
tmp_Var_X0=DQDB_Standard(conc_cal_rep,b1_est,b2_est,b3_est,b4_est)
tmp_Y_Standard=(SDcurve(log(conc_cal_rep),b1_est,b2_est,b3_est,b4_est))^(theta_est)
tmp_Var_X=tmp_Var_X0/(tmp_Y_Standard)
Var_B=(ginv(t(tmp_Var_X)%*%tmp_Var_X))*(sigma_est^2)

#variance estimate for the dilution-specific concentration estimate for the original test sample
for(ki in 1:K)
{
tmpy=od_mat[i,ki]
DlnxDB=c(1/(b1_est-tmpy), 1/(tmpy-b2_est) , b4_est/b3_est, -1/b4_est*log((b1_est-tmpy)/(tmpy-b2_est)))*1/b4_est
DlnxDy=-1/b4_est*(1/(b1_est-tmpy)+1/(tmpy-b2_est))
Var_logx[i,ki]=DlnxDy^2*tmpy^(2*theta_est)*(sigma_est^2)/r0+DlnxDB%*%Var_B%*%DlnxDB
}

wt_subji=1/Var_logx[i,]/sum(1/Var_logx[i,],na.rm=TRUE)
temp_mult=log(conc_mat[i,])*wt_subji
estWtAvg_IV[i]=exp(sum(temp_mult,na.rm=TRUE))


####variance estimation of conconcentration estimate
cov_logx=matrix(0,K,K)
for(ki in 1:K)
{
for(kj in 1:K)
{
if(ki==ki) cov_logx[ki, kj]=Var_logx[i,ki]
if(ki!=kj) {
tmpyi=od_mat[i,ki]
tmpyj=od_mat[i,kj]
DlnxDB_xi=c(1/(b1_est-tmpyi), 1/(tmpyi-b2_est) , b4_est/b3_est, -1/b4_est*log((b1_est-tmpyi)/(tmpyi-b2_est)))*1/b4_est
DlnxDB_xj=c(1/(b1_est-tmpyj), 1/(tmpyj-b2_est) , b4_est/b3_est, -1/b4_est*log((b1_est-tmpyj)/(tmpyj-b2_est)))*1/b4_est
cov_logx[ki,kj]=DlnxDB_xi%*%Var_B%*%DlnxDB_xj
}
}
}
cov_logx[is.na(cov_logx)]=0
var_lnWtAvg_IV[i]=t(wt_subji)%*%cov_logx%*%wt_subji


####estimation of 100(1-alpha)% confidence interval for the conconcentration estimate
#df: degree of freedom of the t distribution (=Ns-4: Ns: total number of replicated standard samples on the same plate; 4: number of parameters in the standard curve)
df=length(conc_cal_rep)-4
Lower_CI_WtAvg_IV[i]=exp(log(estWtAvg_IV[i])-qt(1-alpha/2,df)*sqrt(var_lnWtAvg_IV[i]))
Upper_CI_WtAvg_IV[i]=exp(log(estWtAvg_IV[i])+qt(1-alpha/2,df)*sqrt(var_lnWtAvg_IV[i]))
}


##export estimation result 
data_output=data.frame(subject_mat, plate_mat, estWtAvg_IV, var_lnWtAvg_IV, Lower_CI_WtAvg_IV,Upper_CI_WtAvg_IV)
names(data_output)=c("subjectid", "plateid", "Weight(IV)", "Var(ln(Weight(IV)))", "LowerBound_CI(Weight(IV))","UpperBound_CI(Weight(IV))")
write.csv(data_output,file = "Estimate_IV_output.csv", row.names = FALSE)

